/* exit.c
 * Part of the RTAudio.Client project.
 * This file contains the routines for exiting the program in its different
 * operating modes.
 * Written by Electronic One <eo@autobahn.mb.ca>.
 */

#include <stdio.h>
#include "curses.h"
#include "modes.h"
#include "dialog.h"
#include <unistd.h>  /* For sleep() */

void n_exit(int exitcode)
{
   bkgd(0);
   nocbreak();
   erase();
   clear();
   nl();
   echo();
   curs_set(1);  /* Make cursor normal */
   refresh();
/*   endwin(); */
   exit(exitcode);
}

void t_exit(int exitcode)
{
   printf("\n");
   exit(exitcode);
}

/* P for Program */
void p_exit(char mode, int exitcode)
{
   switch(mode)
     {
      case MODE_TEXT :
	t_exit(exitcode); 
	break;
	
      case MODE_NCURSES : 
	n_exit(exitcode); 
	break;
	
      default : 
	t_exit(exitcode); 
	break;
     }
}

void p_perror(char mode, char *msg)
{
   WINDOW *w;
   
   switch(mode)
     {
      case MODE_NCURSES:
	refresh();
	w = center_message(msg);
	sleep(1);
	delwin(w);
	refresh();
	break;
      case MODE_TEXT:
	perror(msg);
	break;
     }
}


