/* open-close.c
 * Part of the RTAudio.Client project.
 * These routines deal with openning and closing input and output for the
 * RTAudio.Client program.
 * Written by Electonic One <dkaye@cyberspc.mb.ca>
 */

#include "switches.h"
#include "errorcodes.h"
#include "preferences.h"
#include "exit.h"
#include "modes.h"
#include "dialog.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

FILE *open_in(SW *sw)
{
   struct sockaddr_in sa;
   struct hostent *hp;
   int s;
   long addr;
   
   FILE *inf;
   
   if((inf = (FILE *) malloc(sizeof(FILE))) == NULL)
     {
	p_perror(sw->mode, "Insufficient memory");
	p_exit(sw->mode, NO_MEM);
     }
   
   if(sw->useinput)
     {
	if(!strcmp(&sw->inf_name[0], "std"))
	  inf = stdin;
	else
	  if((inf = fopen(&sw->inf_name[0], "rb")) == NULL)
	  {
	     sprintf(&sw->outf_name[0], "Couldn't open '%s' for input", 
		     &sw->inf_name[0]);
	     p_perror(sw->mode, &sw->outf_name[0]);
	     if(sw->mode != MODE_NCURSES)
	       p_exit(sw->mode, ERR_OPEN);
	     else
	       return (FILE *) NULL;
	  }
     }
   
   if(sw->connect)
     {
	bzero(&sa, sizeof(sa));
   
	if((addr = inet_addr(&sw->host_name[0])) != -1)
	  {
	     /* It IS in dotted-quad notation. */
	     bcopy(&addr, (char *) &sa.sin_addr, sizeof(addr));
	     sa.sin_family = AF_INET;
	     if((hp = gethostbyaddr((char*)&sa.sin_addr.s_addr, 
				    sizeof(sa.sin_addr.s_addr), 
				    sa.sin_family)) == NULL)
	       p_perror(sw->mode, "Couldn't resolve host name");
	     else
	       dump_host_info(sw, hp->h_name, hp->h_addr);
	  }
	else
	  {
	     /* It's a hostname: we have to get IP. */
	     if((hp = gethostbyname(&sw->host_name[0])) == NULL)
	       {
		  p_perror(sw->mode, "Couldn't resolve host address");
		  if(sw->mode != MODE_NCURSES)
		    p_exit(sw->mode, NO_RESOLVE);
		  else
		    return (FILE *) NULL;
	       }
	     dump_host_info(sw, hp->h_name, hp->h_addr);
	     bcopy(hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	     sa.sin_family = hp->h_addrtype;
	  }
   
	sa.sin_port = htons(RTA_PORT);
   
	if((s = socket(sa.sin_family, SOCK_STREAM, 0)) < 0)
	  {
	     p_perror(sw->mode, "Couldn't get socket");
	     if(sw->mode != MODE_NCURSES)
	       p_exit(sw->mode, NO_SOCKET);
	     else
	       return (FILE *) NULL;
	  }
   
	if(connect(s, (struct sockaddr *) &sa, sizeof(sa)) < 0)
	  {
	     p_perror(sw->mode, "Couldn't connect to host");
	     close(s);
	     if(sw->mode != MODE_NCURSES)
	       p_exit(sw->mode, NO_CONNECT);
	     else
	       return (FILE *) NULL;
	  }
   
	inf = fdopen(s, "rb");  /* Might wanna change to "rwb" later on */
     }

   
   
   return inf;
}

FILE *open_out(SW *sw)
{
   FILE *outf;
   
   if((outf = (FILE *) malloc(sizeof(FILE))) == NULL)
     {
	p_perror(sw->mode, "Insufficient memory");
	p_exit(sw->mode, NO_MEM);
     }
   
   if(sw->useoutput)
     {
	if(!strcmp(&sw->outf_name[0], "std"))
	  outf = stdout;
	else
	  if((outf = fopen(&sw->outf_name[0], "wb")) == NULL)
	  {
	     sprintf(&sw->inf_name[0], "Couldn't open '%s' for output", 
		     &sw->outf_name[0]);
	     p_perror(sw->mode, &sw->inf_name[0]);
	     if(sw->mode != MODE_NCURSES)
	       p_exit(sw->mode, ERR_OPEN);
	     else
	       return (FILE *) NULL;
	  }
     }
   else
     /* Use the default output if none specified */
     if((outf = fopen("/dev/audio", "wb")) == NULL)
     {
	p_perror(sw->mode, "Couldn't open /dev/audio");
	if(sw->mode != MODE_NCURSES)
	  p_exit(sw->mode, ERR_OPEN);
	else
	  return (FILE *) NULL;
     }

   return outf;
}

void close_in(FILE *inf)
{
   if((inf != stdin) && (inf != NULL))
     fclose(inf);
}

void close_out(FILE *outf)
{
   if((outf != stdout) && (outf != NULL))
     fclose(outf);
}

